@@ -203,19 +203,33 @@ bool readDouble2Inv(RVNGInputStreamPtr &input, double &res, bool &isNaN);
 //! read a double store with 4 bytes: exponent 3.5 bytes, mantisse 0.5 bytes
 bool readDouble4Inv(RVNGInputStreamPtr &input, double &res, bool &isNaN);
 
+//! try to read sz bytes from input and store them in a librevenge::RVNGBinaryData
 bool readData(RVNGInputStreamPtr &input, unsigned long sz, librevenge::RVNGBinaryData &data);
+//! try to read the last bytes from input and store them in a librevenge::RVNGBinaryData
 bool readDataToEnd(RVNGInputStreamPtr &input, librevenge::RVNGBinaryData &data);
+//! adds an unicode character to a string ( with correct encoding ).
+void appendUnicode(uint32_t val, librevenge::RVNGString &buffer);
 }
 
-#define WPS_LE_GET_GUINT16(p)				  \
-        (uint16_t)((((uint8_t const *)(p))[0] << 0)  |    \
+#define WPS_LE_GET_GUINT16(p)				  			\
+        (uint16_t)((((uint8_t const *)(p))[0] << 0)  |	\
                   (((uint8_t const *)(p))[1] << 8))
-#define WPS_LE_GET_GUINT32(p) \
-        (uint32_t)((((uint8_t const *)(p))[0] << 0)  |    \
-                  (((uint8_t const *)(p))[1] << 8)  |    \
-                  (((uint8_t const *)(p))[2] << 16) |    \
+#define WPS_LE_GET_GUINT32(p)				  			\
+        (uint32_t)((((uint8_t const *)(p))[0] << 0) |	\
+                  (((uint8_t const *)(p))[1] << 8)  |	\
+                  (((uint8_t const *)(p))[2] << 16) |	\
                   (((uint8_t const *)(p))[3] << 24))
 
+#define WPS_LE_PUT_GUINT16(p, v)				  		\
+	*((uint8_t*)(p)) = uint8_t(v);				  		\
+	*(((uint8_t*)(p)) + 1) = uint8_t((v) >> 8)
+
+#define WPS_LE_PUT_GUINT32(p, v)				  		\
+	*((uint8_t*)(p)) = uint8_t(v);				  		\
+	*(((uint8_t*)(p)) + 1) = uint8_t((v) >> 8);			\
+	*(((uint8_t*)(p)) + 2) = uint8_t((v) >> 16);		\
+	*(((uint8_t*)(p)) + 3) = uint8_t((v) >> 24)
+
 // Various helper structures for the parser..
 /* ---------- small enum/class ------------- */
 
@@ -400,6 +414,57 @@ struct WPSBorder
 	std::string m_extra;
 };
 
+/** small class use to define a embedded object
+
+    \note mainly used to store picture
+ */
+struct WPSEmbeddedObject
+{
+	//! empty constructor
+	WPSEmbeddedObject() : m_dataList(), m_typeList()
+	{
+	}
+	//! constructor
+	WPSEmbeddedObject(librevenge::RVNGBinaryData const &binaryData,
+	                  std::string type="image/pict") : m_dataList(), m_typeList()
+	{
+		add(binaryData, type);
+	}
+	//! destructor
+	virtual ~WPSEmbeddedObject()
+	{
+	}
+	//! return true if the picture contains no data
+	bool isEmpty() const
+	{
+		for (size_t i=0; i<m_dataList.size(); ++i)
+		{
+			if (!m_dataList[i].empty())
+				return false;
+		}
+		return true;
+	}
+	//! add a picture
+	void add(librevenge::RVNGBinaryData const &binaryData, std::string type="image/pict")
+	{
+		size_t pos=m_dataList.size();
+		if (pos<m_typeList.size()) pos=m_typeList.size();
+		m_dataList.resize(pos+1);
+		m_dataList[pos]=binaryData;
+		m_typeList.resize(pos+1);
+		m_typeList[pos]=type;
+	}
+	/** add the link property to proplist */
+	bool addTo(librevenge::RVNGPropertyList &propList) const;
+	/** operator<<*/
+	friend std::ostream &operator<<(std::ostream &o, WPSEmbeddedObject const &pict);
+
+	//! the picture content: one data by representation
+	std::vector<librevenge::RVNGBinaryData> m_dataList;
+	//! the picture type: one type by representation
+	std::vector<std::string> m_typeList;
+};
+
 namespace libwps
 {
 enum NumberingType { NONE, BULLET, ARABIC, LOWERCASE, UPPERCASE, LOWERCASE_ROMAN, UPPERCASE_ROMAN };
@@ -673,8 +738,7 @@ public:
 	}
 
 	/*! \brief the two extremum points which defined the box
-	 * \param c value 0 means the minimum
-	 * \param c value 1 means the maximum
+	 * \param c value 0 means the minimum, value 1 means the maximum
 	 */
 	Vec2<T> const &operator[](int c) const
 	{
